/* This file contains code and declarations to interface Arena browser
   to the CERN library of common code. Written by howcome@info.cern.ch
   1994 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <sys/param.h>

#include <ctype.h>

#include "HTUtils.h"	/* WWW general purpose macros */
#include "tcp.h"
#include "HTFormat.h"
#include "HTTCP.h"	/* TCP/IP utilities */
#include "HTAnchor.h"   /* Anchor class */
#include "HTParse.h"    /* WWW address manipulation */
#include "HTAccess.h"   /* WWW document access network code */
#include "HTHist.h"	/* Navigational aids */
#include "HTML.h"	/* For parser */
#include "HTFWrite.h"	/* For non-interactive output */
#include "HTMLGen.h"	/* For reformatting HTML */
#include "HTFile.h"	/* For Dir access flags */
#include "HTRules.h"    /* For loading rule file */
#include "HTError.h"
#include "HText.h"
#include "HTEvent.h"	/* multitreading stuff */
#include "HTMIME.h"
#include "HTXParse.h"
#include "HTWSRC.h"
#include "HTThread.h"
#include "HTEvent.h"
#include "HTList.h"
#include "HTString.h"


#include "www.h"

#include "bridge.h"
#include "bridge_decl.h"


HTAtom *text_atom;
HTAtom *html_atom;
HTAtom *html3_atom;
HTAtom *html_level3_atom;
HTAtom *gif_atom;
HTAtom *jpeg_atom;
HTAtom *xpm_atom;
HTAtom *xbm_atom;
HTAtom *png_atom;
HTAtom *png_exp_atom;


PUBLIC char * HTAppName = BANNER;	         /* Application name */
PUBLIC char * HTAppVersion = VERSION; 	              /* Application version */

Doc *PendingDoc;
int prohibit_parsing = 0; /* hack since parser isn't reentrant */

extern Display *display;
extern int screen;
extern Window win;
extern GC gc_fill;
extern Doc *CurrentDoc;
extern long PixelOffset;
extern int debug;
extern char *CacheRoot;
extern int document;
extern int NoMailcap;
extern char *buffer;

extern int initialised;

extern int sbar_width;
extern unsigned int win_width, win_height;
extern int ToolBarHeight;
extern int statusHeight;

extern int buf_width;
extern long buf_height; 	/* if a doc has been fully loaded an parsed, this is > 0 */
extern int PixelIndent;

extern unsigned int HTErrorShowMask; 
extern int AbortFlag;

extern Context *context;
extern int library_trace;

#if 0

BOOL oldDocType(char *a, char *b)
{
    if (!a || !b) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"DocType: called with null pointer \n");
	return FALSE;
    }
    
    if ((strcmp(a,b)) == 0)
	return TRUE;
    return FALSE;
}


BOOL DocType(HTformat a, char *b)
{
    if (!a || !b) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"DocType: called with null pointer \n");
	return FALSE;
    }
    
    if ((strcmp(a,b)) == 0)
	return TRUE;
    return FALSE;
}


BOOL oldDocHTML(char *c)
{
    return( DocType(CurrentDoc->content_type, HTML_DOCUMENT) || DocType(CurrentDoc->content_type, HTML3_DOCUMENT) || DocType(CurrentDoc->content_type,HTML_LEVEL3_DOCUMENT) );
}
#endif

BOOL DocImage(Doc *d)
{
    HTAtom *a;

    if (!d || !d->anchor)
	return False;

    a = HTAnchor_format(d->anchor->parent);

    return(
	   (a == gif_atom) || 
#ifdef JPEG
	   (a == jpeg_atom) || 
#endif
#ifdef PNG
	   (a == png_atom) || (a == png_exp_atom) || 
#endif
	   (a == xpm_atom) ||
	   (a == xbm_atom)
	   );
}

BOOL DocRawText(Doc *d)
{
    HTAtom *a;

    if (!d || !d->anchor)
	return False;

    a = HTAnchor_format(d->anchor->parent);

    if (!d || !d->anchor)
	return False;

    if ((d->show_raw) || (a == text_atom))
	return TRUE;
    else
	return FALSE;
}


BOOL DocHTML(Doc *d)
{
    HTAtom *a;

    if (!d || !d->anchor)
	return True;		/* this being the default type */

    a = HTAnchor_format(d->anchor->parent);

    return((a == html3_atom) || (a == html_level3_atom) || (a == html_atom));
}



Doc *NewDoc()
{
    Doc *c;

    c = ((Doc *) calloc (sizeof(Doc), 1));
    c->state = DOC_NOTREGISTERED;
    return c;
}

void DisplayUrl()
{
    if (CurrentDoc && CurrentDoc->url) {
	if (CurrentDoc->tag)
	    Announce("%s#%s",CurrentDoc->url,CurrentDoc->tag);
	else
	    Announce("%s",CurrentDoc->url);
    }
}


void libSane()
{
    HTList *l, *ll;
    Doc *d;
    HTParentAnchor *a, *aa;

    fprintf(stderr,"=====> LibSane: Checking the anchor and document structures.. this won't hurt\n");

    l = context->pending_docs;
    while(d = (Doc *) HTList_nextObject(l)) {
	fprintf(stderr,"pending_docs: %s\n",d->url);
    }

    l = context->registered_anchors;
    while(a = (HTParentAnchor *) HTList_nextObject(l)) {
#if defined (__alpha) && defined (__osf__)
	fprintf(stderr,"registered_anchors: (%p) %s\n", a, a->physical);
#else
	fprintf(stderr,"registered_anchors: (%lx) %s\n", a, a->physical);
#endif /* (__alpha) && (__osf__) */
	if (d = (Doc *) a->document) {
#if defined (__alpha) && defined (__osf__)
	    fprintf(stderr," document %p\n",d);
#else
	    fprintf(stderr," document %lx\n",d);
#endif /* (__alpha) && (__osf__) */
	    if (ll = d->host_anchors) {
		while(aa = (HTParentAnchor *) HTList_nextObject(ll)) {
		    fprintf(stderr,"    host_docs: %s\n",aa->physical);
		}
	    }
	}
    }

    fprintf(stderr,"=====> Check procedure end\n");
}


void clean_str(char * s)
{
    char * p;

    if (!s)
        return;
    for (p=s; *p!=0; p++) {
	if (!isprint((int)*p))
	    *p=' ';
    }
}

void DocFree(Doc *doc, Bool cut_anchor)
{
    HTList *l;
    Doc *d;

    /* normally, we don't want to free a document if its pending. In
       other cases, the stat flag should be manipulated first */

    if (!doc || doc->nofree || doc->state == DOC_PENDING)
	return;

    if (BRIDGE_TRACE)
	if (doc->url != NULL)
	    fprintf(stderr,"DocFree: freeing %s\n", doc->url);
	else
	    fprintf(stderr,"DocFree\n");
	
/*    Free (doc->href);*/
/*    Free (doc->url); */
/*    Free (doc->request); */ /* problem: if we free this while the doc is still being fetched.. check state.. */
    HTList_addObject(context->memory_leaks, (void *)doc->request);

    Free (doc->content_buffer);
    Free (doc->paint_stream);

    /* make sure we get off these lists

    HTList_removeObject(context->pending_docs, (void *) doc);
    HTList_removeObject(context->pending_anchors, (void *) doc->anchor->parent);


    /* if inline, we have to go through the list of host docs,  .. doing what? nothing! */

    HTList_delete (doc->host_anchors);
    HTList_delete (doc->inline_anchors);
    if(doc->bad_flags)
	HTList_free (doc->bad_flags);

    if (doc->image) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"  freeing pixmap: %d\n", doc->image->pixmap);
	XFreePixmap (display, doc->image->pixmap);
	Free (doc->image);
    }

    /* should we remove it from this list? Probably not..*/
/*    HTList_removeObject(context->registered_anchors, (HyperDoc *) doc);*/

    if (cut_anchor)
	doc->anchor->document = NULL;

    if (doc->style)
	FreeStyleSheet(doc->style);

    Free(doc);
}


void DocsFree(Doc *doc)
{
    HTList *l;
    Doc *d;
    HTParentAnchor *a;

    if (BRIDGE_TRACE && doc->url)
#if defined (__alpha) && defined (__osf__)
	fprintf(stderr,"DocsFree (%p) %s\n",doc, doc->url);
#else
	fprintf(stderr,"DocsFree (%lx) %s\n",doc, doc->url);
#endif /* (__alpha) && (__osf__) */

    if (!doc || doc->nofree)
	return;

    if (doc->url != NULL) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"DocsFree: freeing %s\n", doc->url);
    }
    else {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"DocsFree, no url given\n");
    }

    /* if main doc, we have to go through the inline, remove ourselves
       from the list of hosts, delete the document if we're the last
       main_doc pointing to it */

    l = doc->inline_anchors;
    while (a = (HTParentAnchor *)HTList_nextObject(l)) {
	d = (Doc *)a->document;
	if (d && doc->anchor) {
	    HTList_removeObject(d->host_anchors, doc->anchor);
	    if (HTList_count(d->host_anchors) == 0) {
		DocFree(d, TRUE);
		a->document = NULL;
	    }
	}
    }
    DocFree(doc, TRUE);
    if (BRIDGE_TRACE && VERBOSE_TRACE)
	libSane();
}


/* 
 * The following routines from HTAlert.c replace the ones in the library at link time. 
 */

extern int NoTerminal;

void HTAlert (CONST char * Msg) {
    char *s;

    if (Msg) {
	if (VERBOSE_TRACE)
	    fprintf(stderr,"HTAlert: %s\n",Msg);
	s = strdup(Msg);
	clean_str(s);
	Warn(s);
	Free(s);
    }
}

void HTProgress (CONST char * Msg) {
    char *s;

    if (Msg) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HTProgress\n");
	s = strdup(Msg);
	clean_str(s);
	SetStatusString(s);
	Free(s);
    }
}

BOOL HTConfirm (CONST char * Msg) {
/*
    char *s;


    if (Msg){
	printf("HTConfirm\n");
	s = strdup(Msg);
	clean_str(s);
	SetStatusString(s);
	Free(s);
    }
*/

  char Reply[4];	/* One more for terminating NULL -- AL */
  char *URep;
  
  printf("WWW: %s (y/n) ", Msg);	   /* (y/n) came twice -- AL */
  if (NoTerminal || !fgets(Reply, 4, stdin))   /* get reply, max 3 chars */
      return NO;
  URep=Reply;
  while (*URep) {
    if (*URep == '\n') {
	*URep = (char)0;	/* Overwrite newline */
	break;
    }
    *URep=TOUPPER(*URep);
    URep++;	/* This was previously embedded in the TOUPPER */
                /* call an it became evaluated twice because   */
                /* TOUPPER is a macro -- AL */
  }

  if ((strcmp(Reply,"YES")==0) || (strcmp(Reply,"Y")==0))
    return(YES);
  else
    return(NO);
}

char * HTPrompt (CONST char * Msg, CONST char * deflt) {
    char Tmp[200];
    char * rep = 0;

/*    printf("HTPrompt\n");*/
    printf("WWW: %s", Msg);
    if (deflt) printf(" (RETURN for [%s]) ", deflt);
    if (NoTerminal || !fgets(Tmp, 200, stdin))
	return NULL;		       	/* NULL string on error, Henrik */
    Tmp[strlen(Tmp)-1] = (char)0;	/* Overwrite newline */
   
    StrAllocCopy(rep, *Tmp ? Tmp : deflt);
    return rep;

}

char * HTPromptPassword (CONST char * Msg) {
    char *result = NULL;
    char *pw;

/*    printf("HTPromptPassword\n");*/

    if (NoTerminal)
	return "";
    pw = (char *) getpass(Msg ? Msg : "Password: ");
    StrAllocCopy(result, pw);
    return result;

}

void HTPromptUsernameAndPassword (CONST char * Msg, char ** username, char ** password) {
/*    printf("HTPromptUsernameAndPassword\n");*/

    if (NoTerminal) {
	Window PopUpWin;
	GC PopUpGC;
	int x,y,w,h,text_w,end,count;
	int us_indent, pw_indent;
	char keybuf[20];
	int keybufsize = 20;
	KeySym key;
	XEvent event;
	XComposeStatus cs;
	Field *F_username,*F_password, *current_field;
	char *msg;
	XEvent e;
	EditorBuffer *buffer;
	extern XFontStruct *Fonts[FONTS];
	extern unsigned long textColor, windowColor;

	msg = (strchr(Msg,'.')) ? strchr(Msg,'.')+2 : Msg;
	w = WIDTH(0,msg,strlen(msg)) + 20;
	x = WinWidth/2 + WinLeft - w/2;
	y = WinHeight/2 + WinTop - 70;
	h = 140;
	
	PopUpWin = XCreateSimpleWindow(display, win, x, y, w, h, 0,
				       BlackPixel(display,screen), WhitePixel(display, screen));

	XMapWindow(display, PopUpWin);
	PopUpGC = XCreateGC(display, PopUpWin, 0, NULL);
	XSetFont(display, PopUpGC, Fonts[0]->fid);
/*	XSetClipRectangles(display, PopUpGC, 0, 0, NULL, 0, Unsorted); */
	XSetForeground(display, PopUpGC, rgb2color(0,0,0,0,0));
	XFillRectangle(display, PopUpWin, PopUpGC, 2,2, w-2, h-2);
	XSetForeground(display, PopUpGC, windowColor);
	XFillRectangle(display, PopUpWin, PopUpGC, 2,2, w-3, h-3);
	F_username = (Field *)calloc(1,sizeof(Field));
	F_username->type = TEXTFIELD;
	F_username->flags = CHECKED;
	F_username->width = 100;
	F_username->height = 20;
	F_password = (Field *)calloc(1,sizeof(Field));
	F_password->type = PASSWD;
	F_password->width = 100;
	F_password->height = 20;
	XSetForeground(display, PopUpGC, textColor);
	XDrawString(display,PopUpWin, PopUpGC, 10, 20, msg, strlen(msg));
	text_w = WIDTH(0,"Username:",9);
	us_indent = 2*(w-text_w)/3;
	XDrawString(display,PopUpWin, PopUpGC, (w-text_w)/3, 50,"Username:", 9);
	PaintPopupField(PopUpWin, PopUpGC, us_indent, 50, F_username);
	PaintCursor(PopUpWin, PopUpGC, us_indent,50,F_username, 0);
	text_w = WIDTH(0,"Password:",9);
	pw_indent = 2*(w-text_w)/3;
	XSetForeground(display, PopUpGC, textColor);
	XDrawString(display,PopUpWin, PopUpGC, (w-text_w)/3, 100,"Password:", 9);
	PaintPopupField(PopUpWin, PopUpGC, pw_indent, 100, F_password);
	end = FALSE;
	current_field = F_username;
	buffer = CreateBuffer();
	while (!end)
	{
	    XNextEvent(display, &e);
	    switch(e.type)
	    {
	    case Expose:
		while (XCheckTypedEvent(display, Expose, &e));
		break;
	    case ButtonPress:
		end = TRUE;
		break; 
	    case KeyPress:
		count = XLookupString((XKeyEvent*)&e, keybuf, keybufsize, &key, &cs);
                keybuf[count] = 0;	
		if(key == XK_Return)
		{
		    if(current_field == F_username)
		    {
			free(F_username->value);
			F_username->value = Buffer2Str(buffer);
			F_username->buflen = buffer->size;
			*username = F_username->value;
			current_field = F_password;
			F_username->flags &= ~CHECKED;
			F_password->flags |= CHECKED;
			FreeBuffer(buffer);
			buffer = CreateBuffer();
			PaintPopupField(PopUpWin, PopUpGC, us_indent, 50, F_username);
			PaintPopupField(PopUpWin, PopUpGC, pw_indent, 100, F_password);
			PaintCursor(PopUpWin, PopUpGC, pw_indent,100,F_password, 0);
		    }
		    else
		    {
			free(F_password->value);
			F_password->value = Buffer2Str(buffer);
			F_password->buflen = buffer->size;
			*password = F_username->value;
			end = TRUE;
		    };
		}
		else if (key <255)
		{
		    AppendChar(buffer,key);
		    free(current_field->value);
		    current_field->value = Buffer2Str(buffer);
		    current_field->buflen = buffer->size;
		    ++buffer->pos;
		    PaintPopupField(PopUpWin, PopUpGC, us_indent, 50, F_username);
		    PaintPopupField(PopUpWin, PopUpGC, pw_indent, 100, F_password);
		    if(current_field == F_username)
			PaintCursor(PopUpWin, PopUpGC, us_indent,50,F_username, buffer->pos);
		    else
			PaintCursor(PopUpWin, PopUpGC, pw_indent,100,F_password, buffer->pos);
		}
	    }
	}
	XUnmapWindow(display, PopUpWin);
	XDestroyWindow(display, PopUpWin);
	XFreeGC(display, PopUpGC);
	return;
    }
    if (Msg)
	printf("WWW: %s\n", Msg);
    *username = HTPrompt("Username: ", *username);
    *password = HTPromptPassword("Password: ");

}


void HTErrorMsg (HTRequest * request)
{
    HTErrorInfo *e;
    HTList *l = request->error_stack;

    fprintf(stderr,"HTErrorMsg\n");

    while (e = (HTErrorInfo *)HTList_nextObject(l)) {
	if (e->par) {
	    char *s = strndup(e->par, e->par_length);
	    Warn(s);
	    Free(s);
	    fprintf(stderr,"message: %s\n",e->par);
	}
    }	
}



/*
 * the following routines HAVE to be defined 
 */

PUBLIC void HText_beginAppend(HText *text) { 
}

PUBLIC void HText_endAppend(HText *text) { 
}

PUBLIC void HText_appendParagraph(HText *text) { 
}

PUBLIC void HText_appendCharacter(HText *text, char ch) { 
}

PUBLIC BOOL HText_selectAnchor(HText *text, HTChildAnchor *anchor) { 
    return TRUE;
}

PUBLIC void HText_appendImage(HText *text, HTChildAnchor *anc, CONST char *alt, CONST char *alignment, BOOL isMap){
}

PUBLIC void HText_beginAnchor(HText *text, HTChildAnchor *anc) { 
}

PUBLIC void HText_endAnchor(HText *text) {
}

PUBLIC BOOL HText_select(HText * text) { 
    return TRUE;
}

PUBLIC void HText_appendText(HText * text, CONST char * str) {
}

PUBLIC HText* HText_new2(HTParentAnchor *anchor, HTStream *output_stream) { 
    return NULL;
}


PUBLIC void HText_setStyle(HText *text, HTStyle *style) { 
}


/* procedure to register inline images, referenced style sheets etc. */

Doc *GetInline(char *href, int hreflen, BOOL reload)
{
    XGCValues values;
    Image *image;
    int tag;
    HTList *l;
    HTAnchor *a;
    Doc *doc = NULL;

    /* check for null name */

    if (href == NULL || hreflen == 0)
    {
        Warn("Missing or bad inline name");
        return NULL;
    }

    /* first, lets expand the href into an anchor */

    a = libExpandHref(href, hreflen);

    /* check if document is already in place */

    if (a->parent->document && !CurrentDoc->pending_reload) {
	doc = (Doc *)a->parent->document;
    }
    else {

	/* no, the document does not seem to be in place, lets register it.. */

	if (BRIDGE_TRACE) {
	    char *s = strndup(href, hreflen);
	    fprintf(stderr,"GetInline registering %s\n",s);
	    Free(s);
	}
	
	libLoadAnchor(a, CurrentDoc, FALSE, FALSE, TRUE, reload, FALSE);

	/* if image is still not there.. */

	if (!a->parent->document) {
	    return NULL;
	} else
	    doc = (Doc *) a->parent->document;
    }

    if (doc) {

	/* the inline doc has been registered, it can be loading, loaded, processed*/

	switch (doc->state) {

	  case DOC_PENDING:
	    if (BRIDGE_TRACE) {
		char *s = strndup(href, hreflen);
		fprintf(stderr,"doc has been registered, but not loaded -> returning default%s\n",s);
		Free(s);
	    }
	    return NULL;
	    break;

	  case DOC_PROCESSED:
	    if (BRIDGE_TRACE)
		fprintf(stderr,"GetImage: doc processed returning image %s\n",doc->url);
	    return doc;
	    break;

	  case DOC_LOADED:
	    return doc;
	}
    }
    return NULL;
}


HTAnchor *libFindAnchor(HTRequest *req)
{
    HTAnchor *a;
    HTList *l = context->registered_anchors;

    if (!req)
	return NULL;

    if (req->childAnchor)
	return (HTAnchor *) req->childAnchor;
    else if (req->anchor)
	return (HTAnchor *) req->anchor;

    while (a = (HTAnchor *)HTList_nextObject(l)) {
	if (((HTChildAnchor *)a == req->childAnchor) || ((HTParentAnchor *)a == req->anchor)) 
	    return a;
    }	

/*    fprintf(stderr,"libFindAnchor: request structure had no anchor info \n");*/
    return NULL;
}

HTParentAnchor *libFindAnchorInList(HTRequest *req)
{
    HTParentAnchor *a;
    HTList *l = context->registered_anchors;

    while (a = (HTParentAnchor *)HTList_nextObject(l)) {
	if (((HTChildAnchor *)a == req->childAnchor) || ((HTParentAnchor *)a == req->anchor)) 
	    return a;
    }
    return NULL;
}

Doc *libFindDoc(HTRequest *req)
{
    Doc *d;
    HTList *l = context->pending_docs;

    while (d = (Doc *)HTList_nextObject(l)) {
	if (CONNECT_TRACE)
	    fprintf(stderr,"libFindDoc %s\n",d->url);
	if (d->request == req) {
/*	    HTList_removeObject(l, d); */
	    return d;
	}
    }	
    return NULL;
}



void ConnectAnchorDocument(HTAnchor *anchor, HTRequest *request, BOOL terminate)
{
    Doc *doc;

    /* if the document is being reloaded, the anchor will be pointing
       to a document stucture. NULL it out so that we will enter the
       next if. CurrentDoc is still pointing to it so that it can be
       freed later */

    if (CONNECT_TRACE)
	fprintf(stderr,"\nConnectAnchorDocument: trying to connect %s\n",anchor->parent->address);

    if (anchor->parent->document) {
      	Doc *d = (Doc *)anchor->parent->document;
	if (d->pending_reload) {
	    if (CONNECT_TRACE)
		fprintf(stderr,"ConnectAnchorDocument: %s is marked for death\n",d->url);
	    anchor->parent->document = NULL; /* null it so that we will enter the next if */
	}
    }

    /* in case we never came through a call to HTCallClient, the
       document and anchor structure are not connected. So do it */


    /* watch out! We're about to connect the document structure with
       the anchor structure */

    if (!anchor->parent->document) {
	doc = libFindDoc(request);
	anchor->parent->document = (HyperDoc *) doc;
	if (doc) {
	    if (CONNECT_TRACE)
		fprintf(stderr,"ConnectAnchorDocument: success && terminate %s\n",doc->url);

	    if (BRIDGE_TRACE)
#if defined (__alpha) && defined (__osf__)
		fprintf(stderr,"---->HTCallClinentl deregistering pending %p\n", anchor);
#else
		fprintf(stderr,"---->HTCallClinentl deregistering pending %lx\n", anchor);
#endif /* (__alpha) && (__osf__) */

	}

	/* if we havn't been able to connect at this point, we it's
           because the document has been freed. No need to worry */
    }
}



/* HTCheckPresentation is used to decide if a content type should be
   registered or not. If the content type is not registered before,
   return TRUE to hafe it registered. If the content type has been
   registered, check the quality associated with it. */


BOOL HTCheckPresentation(HTList * conversions, char * content_type, double quality)
{
    HTList *l = conversions;
    HTPresentation *p;

    while (p = (HTPresentation *)HTList_nextObject(l)) {
	if (strcmp(content_type, p->rep->name) == 0) {
	    if (p->quality < quality)
		return TRUE;
	    else {
		if (MAILCAP_TRACE)
		    fprintf(stderr,"HTCheckPresentation returning FALSE for %s\n",content_type);
		return FALSE;
	    }
	}
    }
    return TRUE;
}

    
int HTCallClient(HTExtParseStruct * eps)
{
    char s[80];
    Doc *doc = NULL;
    HTAnchor *anchor;

    /* the ftp module is not yet multithreaded, so we use this opportunity to handle some X Events */

    GuiEvents(0, NULL, 0);
    GuiEvents(0, NULL, 0);
    GuiEvents(0, NULL, 0);

    /* if anchor is null at this point, it has not yet been
       registered-- maybe we're dealing with a blocking i/o operation
       -- e.g. local file */

    anchor = libFindAnchor(eps->request);

    if (anchor == NULL){
	fprintf(stderr,"HTCallClient: NO ANCHOR FOUND, returning\n");
	return EVENT_INTR;
    }

    ConnectAnchorDocument(anchor, eps->request, FALSE);

    if (!anchor->parent->document) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HTCallClient: no document struct can be found for %s, probably freed\n",anchor->parent->address);
	return EVENT_OK;
    }

    doc = (Doc *) anchor->parent->document;
    doc->url = anchor->parent->address;

    if (eps->finished) {
	doc->request = eps->request;
	eps->buffer[eps->used] = 0;   /* NULL-terminating this seems necessary.. do we cross boundaries?? */
	doc->content_buffer = eps->buffer;

	/* we're about to determine the content type of the
           document. Ideally this infomation should be stored in the
           anchor structure. It's not. If the document has come though
           the stream stack the normal way, it will be set in the eps
           stucture. If it's of type gopher etc. it will be set in the
           request structure */

#if 0
	if (!doc->content_type) {
	    if (eps->request->content_type)
		doc->content_type = eps->request->content_type->name;
	    else
		doc->content_type = eps->content_type;
	}

	if (!doc->content_type) {
	    if (BRIDGE_TRACE)
		fprintf(stderr,"HTCallClient: unknown content_type\n");
	    doc->content_type = HTML_DOCUMENT; /* what else can we do? */
	}

	doc->content_length = eps->used;
#endif
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HTCallClient finishing %s, got %d bytes\n",doc->url,eps->used);

    } else {
	if (BRIDGE_TRACE && VERBOSE_TRACE) {
	    fprintf(stderr,"HTCallClient %s, read %d bytes %lx\n",
		    doc->url, eps->used, doc->anchor);
	}
	doc->state = DOC_PENDING;
	doc->loaded_length = eps->used;

	if (initialised){
	    if (doc->loaded_length > 1024) {
		long int len = HTAnchor_length(doc->anchor->parent);
		int a, b;

		/* first, compute the k lengths */

		a = (doc->loaded_length + 512) / 1024;
		b = (len + 512) / 1024;

		if (len > 0) {     /* if content-length was given */
		    int left = len - doc->loaded_length;

		    if (a == b && left > 0) {
			Announce("Read %dk (%d bytes remain) from %s", a, left, doc->url);	
		    } else {
			Announce("Read %dk/%dk from %s", a, b, doc->url);
		    }
		} else
		    Announce("Read %dk of %s", (int)((doc->loaded_length + 512) / 1024), doc->url);
	    }
	}
    }
    return EVENT_OK;
}


PRIVATE	HTList *	reqlist = NULL;	


/*
**  This function creates a new request structure and adds it to the global
**  list of active threads
*/
HTRequest *Thread_new (BOOL Interactive)
{
    HTRequest *newreq = HTRequest_new(); 	     /* Set up a new request */
/*
    if (!reqlist)
	reqlist = HTList_new();
    HTList_addObject(reqlist, (void *) newreq);
*/

/*    newreq->output_format = WWW_SOURCE;*/
    newreq->output_format = NULL;
    newreq->output_stream = HTExtParse(newreq, NULL, NULL, NULL, NULL);

    newreq->conversions = context->conversions;
    return newreq;
}


/*
**  This function deletes a request structure and takes it out of the list
**  of active threads.
*/
void Thread_delete(HTRequest * oldreq)
{
    if (oldreq) {
	if (reqlist)
	    HTList_removeObject(reqlist, (void *) oldreq);
	HTRequest_delete(oldreq);
    }
}

/*
**  This function deletes the whole list of active threads.
*/
void Thread_deleteAll()
{
    if (reqlist) {
	HTList *cur = reqlist;
	HTRequest* pres;
	while ((pres = (HTRequest *) HTList_nextObject(cur)) != NULL)
	    HTRequest_delete(pres);
	HTList_delete(reqlist);
	reqlist = NULL;
    }
}

/*						        HTEventRequestTerminate
**
**	React to the status of the terminated request
*/

HTEventState HTEventRequestTerminate(HTRequest * req, int status) 
{
    char *s;
    Doc *d,*doc = NULL;
    HTAnchor *anchor, *old_anchor;
    HTList *l;
    History *h;
    char *tag = NULL;


/* At this point, there may have been a document redirection. We have
   a pointer to the old anchor structure in the Doc record. Do we? Where is it..? */

    anchor = libFindAnchor(req);

    if (!anchor) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HTEventRequestTerminate: no anchor found\n");
	return EVENT_INTR;
    }

    if (anchor != (HTAnchor *)anchor->parent) {
	HTChildAnchor *c = (HTChildAnchor *)anchor;
	tag = c->tag;
    }

    ConnectAnchorDocument(anchor, req, TRUE);

    doc = (Doc *) anchor->parent->document;

    HTList_removeObject(context->pending_docs, (void *) doc);
    HTList_removeObject(context->pending_anchors, (void *) anchor->parent);

    if (doc == NULL){
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HTEventRequestTerminate: doc has not been registered\n");
	return EVENT_INTR;
    }

    if (anchor->parent != doc->anchor) {
	
	/* a redirection has taken place -- replace the occurences of
           the old_anchor with the new one */

	if (doc->anchor == context->home_anchor)
	    context->home_anchor = anchor->parent;

	l = context->history;

	while (h = (History *)HTList_nextObject(l)) {
	    if (h->anchor->parent == doc->anchor) {
		h->anchor = (HTAnchor *)anchor;  /* should we cast here ?? */

		if (HISTORY_TRACE)
		    fprintf(stderr,"HTEventRequestTerminate: changing pointer to new anchor\n");
	    }
	}
	doc->anchor = anchor->parent;
    }

/*
    if (req->redirections) {
	if (HISTORY_TRACE)
	    fprintf(stderr,"HTEventRequestTerminate: redirection !!\n");
    }	
*/

    doc->url = anchor->parent->address;

    if (BRIDGE_TRACE)
	fprintf(stderr,"HTEventRequestTerminate %s\n", doc->url);

    switch (status) {
      case HT_LOADED:
	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    fprintf(stderr,"    HTEventRequestTerminate HT_LOADED\n");

	if (doc->state < DOC_LOADED)
	    doc->state = DOC_LOADED;

	if (doc->main_doc) {

	    /* we will get to this point even when the document was
               handled by an external application registered through
               mailcap. The only (?) way to detect this is if
               content_buffer is NULL */

	    if (doc->content_buffer == NULL) {
		HideBusy();
		HistoryDelete((HTAnchor *)doc->anchor);
		DocsFree(doc);
		return EXIT_SUCCESS;
	    }

	    if (BRIDGE_TRACE) {
#if defined (__alpha) && defined (__osf__)
		fprintf(stderr,"    HTEventRequestTerminate register: %s %p\n", doc->url, doc->anchor);
#else
		fprintf(stderr,"    HTEventRequestTerminate register: %s %lx\n", doc->url, doc->anchor);
#endif /* (__alpha) && (__osf__) */
	    }


	    if (doc != CurrentDoc) {
		if (doc->anchor == CurrentDoc->anchor) {
		    doc->pending_reload = CurrentDoc->pending_reload;
		    doc->nofree = CurrentDoc->nofree;
		    doc->user_style = CurrentDoc->user_style;
		}

		DocsFree(CurrentDoc);
		CurrentDoc = doc;

		if (CurrentDoc->anchor->parent == context->home_anchor->parent)
		    CurrentDoc->user_style = TRUE;

	    }
	    
	    CurrentDoc->tag = tag;

	    /* at this point, we should verify the history
               registeration done in libGetDocument */

	    HistoryVerify(anchor);


	    /* if an external view shows an image, the content_buffer will be NULL */

	    if (!CurrentDoc->content_buffer)
		CurrentDoc->loaded_length = 0;

	    NewBuffer(doc);

	    /* if inlines came in during the call to NewBuffer (which
               may call libGetDocument, which may call
               HTEventRequestTerminate etc.)  we need to do another
               parse to get those documents in */

	    if (prohibit_parsing == 2) { /* if we earlier have been prohibited to parse */
		NewBuffer(doc);
		prohibit_parsing = 0;
	    }

	    /* if the document hasn't been processed by now, it never
	       will be. Since we've already changed CurrentDoc, we need to change it
	       back. This is easiest with BackDoc() */

	    if (doc->state != DOC_PROCESSED) {
		BackDoc();
	        HistoryDelete(anchor);
		Announce("Unable to display %s",doc->url);
		HideBusy();
		return EVENT_OK;
	    }      

	    /* if the document has been reloaded, turn off flag to
	       avoid looping with inlines */

	    CurrentDoc->pending_reload = FALSE;

	    if (!CurrentDoc->tag && context->current_history->y)
		DeltaHTMLPosition(context->current_history->y);
	    DisplayDoc(WinLeft, WinTop, WinWidth, WinHeight);
	    if (!CurrentDoc->tag) {
		SetScrollBarHPosition(PixelIndent, buf_width);
		SetScrollBarVPosition(context->current_history->y, buf_height);
	    }
	    DisplayScrollBar();
	    DisplayUrl();
	    HideBusy();

	} else {

	    /* this is an inline finishing -- first check to see that
               the host_anchors exists, in it the hostimg document
               should be found */

	    if (doc->host_anchors) {
		if (BRIDGE_TRACE) {
		    fprintf(stderr,"    HTEventRequestTerminate EVENT_OK inline\n");
		}

		/* if this is the second HTEventRequestTerminate on
                   the stack, we must not reenter the parsing routines */

		if (prohibit_parsing == 0) {
		    NewBuffer(CurrentDoc);
		    if (!CurrentDoc->tag)
			DeltaHTMLPosition(context->current_history->y);
		    DisplayDoc(WinLeft, WinTop, WinWidth, WinHeight);
		    if (!CurrentDoc->tag) {
			SetScrollBarHPosition(PixelIndent, buf_width);
			SetScrollBarVPosition(context->current_history->y, buf_height);
		    }
		    DisplayScrollBar();
		    DisplayUrl();

		} else
		    if (TAG_TRACE)
			fprintf(stderr,"HTEventRequestTerminate: parsing prohibited\n");

	    } else
		if (BRIDGE_TRACE)
		    fprintf(stderr,"    HTEventRequestTerminate: WHOOPS no host_doc specified\n");

	    if (CurrentDoc)
		DisplayUrl();
	}
	return EVENT_OK;
	break;

      case HT_WOULD_BLOCK: 
	fprintf(stderr,"    HTEventRequestTerminate HT_WOULD_BLOCK, what does this mean here????\n");
	DocFree(doc, TRUE);
	HideBusy();
	return EVENT_OK;
	break;

      case HT_ERROR:
	/* we end up here when the library finds no document */
	if (BRIDGE_TRACE)
	    fprintf(stderr,"    HTEventRequestTerminate HT_ERROR\n");
	if (doc->main_doc) {
	    HideBusy();
	    HistoryDelete((HTAnchor *)doc->anchor);
	}

	DocFree(doc, TRUE);
	HideBusy();
	return EVENT_INTR;
	break;

      case HT_NO_DATA:
	if (BRIDGE_TRACE)
	    fprintf(stderr,"    HTEventRequestTerminate HT_NO_DATA\n");
	DocFree(doc, TRUE);
	HideBusy();
	return EVENT_OK;
	break;
    }
    return EXIT_SUCCESS; /* wm 18.Jan.95 */
}


HTEventState libCallback(HTRequest ** req)
{
    HTParentAnchor *anchor;
    Doc *doc;

    GuiEvents(0, NULL, 0);
    GuiEvents(0, NULL, 0);

    if (*req) {
/*	anchor = libFindAnchorInList(*req);*/
	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    fprintf(stderr,"libCallback, %s\n", (*req)->anchor->parent->address);
    }

    if (AbortFlag) {	/* we'll have to abort this session */
	HTList *l;
	Doc *d;

	if (BRIDGE_TRACE)
	    fprintf(stderr,"libcallback: aborting req\n");
	AbortFlag = 0;
	HideBusy();
	DisplayUrl();

	/* we are about to close all connections, so we should empty
           the lists of pending anchors/documents */

	l = context->pending_docs;

	while (d = (Doc *)HTList_nextObject(l)) {
	    d->state = DOC_ABORTED;
	    if (d->anchor && d->anchor->parent->cacheItems)
		HTCacheClear(d->anchor->parent->cacheItems);
	    DocsFree(d);
	}

	HTList_delete(context->pending_docs);
	context->pending_docs = HTList_new();

	HTList_delete(context->pending_anchors);
	context->pending_anchors = HTList_new();

	return EVENT_INTR_ALL; /* interrupt everything !! */
    }
    else {
	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    fprintf(stderr,"libcallback EVENT_OK\n");
	return EVENT_OK;
    }
}


Doc *DocNew(HTRequest *req)
{
    Doc *d;

    if (req == NULL)
	return NULL;

    d = (Doc *) calloc (sizeof(Doc),1);

    if (d==NULL)
	return NULL;

    return d;
}

void libEventLoop(int c, char *href)
{
    HTEventCallBack ecb;
    char * ref;
    Doc *doc;
    HTAnchor *anchor;
    HTParentAnchor *p_anchor;
    HTRequest *request;
    Doc *init_doc; /* must be visiable to style.c to determine if inital doc is being loaded */

    /* register socket for X events */

    ecb.sockfd = c;
    ecb.callback = libCallback;
    HTEventRegister(&ecb);

    /* display initial page */

    init_doc = NewDoc(); /* free this at some point */
    init_doc->main_doc = TRUE;
    init_doc->url = NULL;
    init_doc->nofree = TRUE;
    init_doc->content_buffer = INITIAL_HTML;
    init_doc->tag = NULL;
    init_doc->already_displayed = FALSE;
    init_doc->source_editor = NULL;
    init_doc->field_editor = NULL;
    init_doc->edited_field = NULL;

    /* get atom values, so that we don't have to do strcmp for content types later */

    text_atom = HTAtom_caseFor(TEXT_DOCUMENT);
    html_atom = HTAtom_caseFor(HTML_DOCUMENT);
    html3_atom = HTAtom_caseFor(HTML3_DOCUMENT);
    html_level3_atom = HTAtom_caseFor(HTML_LEVEL3_DOCUMENT);
    gif_atom = HTAtom_caseFor(GIF_DOCUMENT);
    jpeg_atom = HTAtom_caseFor(JPEG_DOCUMENT);
    png_atom = HTAtom_caseFor(PNG_DOCUMENT);
    png_exp_atom = HTAtom_caseFor(PNG_EXP_DOCUMENT);
    xpm_atom = HTAtom_caseFor(XPM_DOCUMENT);
    xbm_atom = HTAtom_caseFor(XBM_DOCUMENT);

    CurrentDoc = init_doc;
    NewBuffer(init_doc);
	
    DisplayAll();
    
    GuiEvents(0, NULL, 0);
    GuiEvents(0, NULL, 0);
    GuiEvents(0, NULL, 0);


    /* now do home page */

    anchor = libExpandHref(href, strlen(href));
/*    p_anchor = anchor->parent; */

    context->home_anchor = anchor->parent;
    libLoadAnchor(anchor, NULL, TRUE, TRUE, TRUE, FALSE, FALSE);

    /* enter infinete loop */

    request = Thread_new(YES);
    HTEventLoop(request, NULL, NULL);
    fprintf(stderr,"libeventloop exiting, why??\n");
    Free(ref);
}

/*
 * this function is the arena frontend to HTLoad
 *
 * href
 * hreflen
 * who		currently unused
 * main_doc	is this a main document or an inline referenced from a host_doc
 * host_doc	if inline, this points to host document
 */

HTAnchor *libExpandHref(char *href, size_t hreflen) /* wm 18.Jan.95 */
{
    int c;
    char *ref;
    char *hrefdup = NULL;
    HTChildAnchor *child_anchor;
    HTParentAnchor *parent_anchor;
    HTAnchor *anchor;

    if (strncmp(href,"mailto:",7) == 0) { 
	Announce("mailto not supported: %s",href);
	return NULL;
    }

    hrefdup = strndup(href,hreflen);

    if (CurrentDoc && CurrentDoc->base) {
	ref = HTParse(hrefdup, CurrentDoc->base, PARSE_ALL);
    } else if (CurrentDoc && CurrentDoc->url) {
	ref = HTParse(hrefdup, CurrentDoc->url, PARSE_ALL);
    } else {
	ref = HTParse(hrefdup, (char *)HTFindRelatedName(), PARSE_ALL);
    }

    if (BRIDGE_TRACE)
	fprintf(stderr,"---->libExpandHref expanding %s into  %s\n", hrefdup, ref);

    anchor = (HTAnchor *) HTAnchor_findAddress(ref); /* we have to free anchor->address */
    child_anchor = (HTChildAnchor *) anchor;
    parent_anchor = child_anchor->parent;

    Free(ref);
    
    if (parent_anchor->document) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"libExpandHref: document structure for %s already in memory\n",hrefdup);
	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    libSane();
    }

    Free(hrefdup);    
    return anchor;
}

char *ExpandHref(char *href, size_t hreflen)
{
    int c;
    char *ref;
    char *hrefdup = NULL;
    HTChildAnchor *child_anchor;
    HTParentAnchor *parent_anchor;
    HTAnchor *anchor;

    hrefdup = strndup(href,hreflen);

    if (CurrentDoc && CurrentDoc->base) {
	ref = HTParse(hrefdup, CurrentDoc->base, PARSE_ALL);
    } else if (CurrentDoc && CurrentDoc->url) {
	ref = HTParse(hrefdup, CurrentDoc->url, PARSE_ALL);
    } else {
	ref = HTParse(hrefdup, (char *)HTFindRelatedName(), PARSE_ALL);
    }

    if (BRIDGE_TRACE)
	fprintf(stderr,"---->libExpandHref expanding %s into  %s\n", hrefdup, ref);
    
    return(ref);
}

void libLoadAnchor(HTAnchor *anchor, Doc *host_doc, BOOL main_doc, BOOL nofree, BOOL record, BOOL reload, BOOL block)
{
    int c;
    Doc *doc;
    char * ref;
    char *hrefdup = NULL;
    int status;
    HTRequest *request;
    char *adr = anchor->parent->address;

    if (main_doc && record)
	HistoryRecord(anchor);

    /* we've been asked to load an anchor first make sure that the anchor isn't pending */

    if (HTList_indexOf(context->pending_anchors, (void *)anchor->parent) >= 0) {
	if (BRIDGE_TRACE)
	    fprintf(stderr,"libLoadAnchor: Anchor pending %lx\n",anchor);
	if (main_doc)
	    Announce("Waiting for: %s", adr);
	return;
    }

    Announce("Fetching %s", adr);
   
    if (main_doc) {
	ShowBusy();
    }

    request = Thread_new(YES);
    request->ForceReload = reload;

    if ((strncmp(adr,"news:",5) == 0) ||
	(strncmp(adr,"nntp:",5) == 0) ||
	(strncmp(adr,"wais:",5) == 0) ||
	(strncmp(adr,"ftp:",4) == 0) ||
	(strncmp(adr,"gopher:",7) == 0)) {
	request->output_format = WWW_SOURCE;
    }

    doc = NewDoc();

    doc->request = request;
    doc->anchor = anchor->parent;
    if(strchr(adr,':'))
	doc->href = strchr(strchr(adr,':')+3,'/');   
    /* register the document */ 
    
    if (!main_doc && host_doc) {

	/* first, put the inline in the host_doc's list of inlines */

	if (!host_doc->inline_anchors)
	    host_doc->inline_anchors = HTList_new();
	HTList_addObject(host_doc->inline_anchors, (void *) doc->anchor);
	
	/* then, put the host_doc in the inline's list of host_docs */

	if (!doc->host_anchors)
	    doc->host_anchors = HTList_new();
	HTList_addObject(doc->host_anchors, (void *) host_doc->anchor);
    }
    
    doc->main_doc = main_doc;
    doc->nofree = nofree;
    doc->request->conversions = context->conversions;
    doc->request->BlockingIO = block;
    doc->url = anchor->parent->address;

    if (main_doc) {
	if (BRIDGE_TRACE) {
	    fprintf(stderr,"---->libLoadAnchor registering maindoc: %s\n", doc->url);
	}
    }

    doc->state = DOC_PENDING;
    HTList_addObject(context->pending_docs, (void *) doc);
    HTList_addObject(context->pending_anchors, (void *) anchor->parent);
    if (BRIDGE_TRACE)
#if defined (__alpha) && defined (__osf__)
	fprintf(stderr,"---->libLoadAnchor registering pending: %p\n", anchor->parent);
#else
	fprintf(stderr,"---->libLoadAnchor registering pending: %lx\n", anchor->parent);
#endif /* (__alpha) && (__osf__) */
   

    PendingDoc = doc;
    status = HTLoadAnchor((HTAnchor*) anchor, doc->request);
    PendingDoc = NULL;

    /* context->registered_anchors contain a list of all anchor
       structures we have allocated. It is due to be registered,
       unless is already in memory. This we can only find out when we
       have access to the anchor
       */

    switch (status) {
      case HT_WOULD_BLOCK: 

	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    fprintf(stderr,"libLoadAnchor: HT_WOULD_BLOCK, fine, register_anchor %lx\n",anchor->parent);
	HTList_addObject(context->registered_anchors, (void *) anchor);
	/* return EVENT_OK; */
	break;

      case HT_ERROR:
	if (BRIDGE_TRACE)
	    fprintf(stderr,"HT_ERROR\n");
	HideBusy();
	Warn("Error fetching %s", anchor->parent->address);
	HTList_removeObject(context->pending_anchors, (void *) anchor->parent);

	/* return EVENT_TERM; */
	return;
	break;

      case HT_LOADED:

	/* at this point we know that the document is loaded, but it's
           unclear if it was already in memory before the HTLoadAnchor
           (in which case the document structure is set up and we need
           to free the newly created doc structure) or if it was
           loaded during the call to HTLoadAnchor (in which case we we
           will use the newly created doc structure) */

	if (BRIDGE_TRACE && VERBOSE_TRACE)
	    fprintf(stderr,"libLoadAnchor: HT_LOADED\n");

	HTList_removeObject(context->pending_anchors, (void *) anchor->parent);

	/* if the anchor has a document attached to it, we known that
	   the document structure has been properly set up and we
	   should discard the newly created document structure
	   */

	if (anchor->parent->document  && ((Doc *)anchor->parent->document != doc)) {  /* is the document structure in place ?? */
	    Doc *d = doc;

	    doc = (Doc *) anchor->parent->document;

	    /* before discaring the structure, we need to save the
               value of href, since this is GetImage's only way of
               finding it.. or ?? */ /* or, could libGetDocumetn return the anchor ?? */

/*	    Free(doc->href);*/
/*	    doc->href = strdup(d->href);*/

	    /* discard the structure, but don't mess up the anchor it's poiting to */
	    
	    DocFree(d, FALSE); /* does this also free request structure that we use next ?? */ /* does it fre current doc we use next */
	    HTList_removeObject(context->pending_docs, (void *) d);

	}

	prohibit_parsing = 0;
	
	/* since libLoadAnchor may be called from within a parse,
	   and the parse routines are not reentrant, we have to
	   flag that no parsing should be done for when inline
	   docs finishes */
	
	if (!main_doc) {
	    prohibit_parsing = 1;
	}

	/* we need to record the anchor in the request structure, since this is later used to identify the anchor. */

	if ((HTAnchor *)anchor->parent == anchor) {
	    doc->request->anchor = (HTParentAnchor *) anchor;
	    doc->request->childAnchor = NULL;
	} else  {	/* child anchor */
	    doc->request->anchor = (HTParentAnchor *) anchor->parent;
	    doc->request->childAnchor = (HTChildAnchor *) anchor;
	}

	anchor->parent->document = (HyperDoc *)doc;

	/* artificial synthetic call */
	/* the articifial call causes a lot of trouble -- rather do Henrik's return scheme? */

	HTEventRequestTerminate(doc->request, HT_LOADED);
	if (prohibit_parsing == 1) /* if parsing was probibited, we have to do it later */
	    prohibit_parsing = 2;

	/* return EVENT_TERM; */
	break;
      case HT_NO_DATA:
	fprintf(stderr,"HT_NO_DATA\n");
	return;
	break;
    }
    return;
}

HTAnchor *libGetDocument(char *href, size_t hreflen, Doc *host_doc, BOOL main_doc, BOOL nofree, BOOL record, BOOL reload, BOOL block)
{
    HTAnchor *anchor, *a;
    HTList *l;

    anchor = libExpandHref(href, hreflen);

    if (!anchor)
	return NULL;

    if (reload) {
/*	HTForceReload = YES;*/ /* howcome 11/5/95: taken out and replaced by entry in the request structure request->ForceReload */
	CurrentDoc->pending_reload = TRUE;
    }
    
    libLoadAnchor(anchor, host_doc, main_doc, nofree, record, reload, block);
/*    HTForceReload = NO;*/
    return anchor;
}


void libEntry(int c)
{
    WWW_TraceFlag = library_trace; /* turn on debugging in Library */

    HTLibInit();
    
    HTErrorShowMask = HT_ERR_SHOW_DEFAULT; /*  ^  HT_ERR_SHOW_FATAL; */ /* no fatal */ 

    HTSetConversion(context->conversions, "www/mime",       	"*/*",            HTMIMEConvert, 1.0, 0.0, 0.0);
    HTSetConversion(context->conversions, TEXT_DOCUMENT,    	"www/present",    HTExtParse,    0.4, 0.0, 0.0);
    HTSetConversion(context->conversions, GIF_DOCUMENT,     	"www/present",    HTExtParse,    0.6, 0.0, 0.0);
#ifdef JPEG
    HTSetConversion(context->conversions, JPEG_DOCUMENT,     	"www/present",    HTExtParse,    0.8, 0.0, 0.0);
#endif /* JPEG */
#ifdef JPEG
    HTSetConversion(context->conversions, PNG_DOCUMENT,     	"www/present",    HTExtParse,    0.8, 0.0, 0.0);
    HTSetConversion(context->conversions, PNG_EXP_DOCUMENT,    	"www/present",    HTExtParse,    0.8, 0.0, 0.0);
#endif /* JPEG */
    HTSetConversion(context->conversions, XPM_DOCUMENT,		"www/present",    HTExtParse,    0.6, 0.0, 0.0);
    HTSetConversion(context->conversions, XBM_DOCUMENT, 	"www/present",    HTExtParse,    0.6, 0.0, 0.0);
    HTSetConversion(context->conversions, HTML_DOCUMENT,      	"www/present",    HTExtParse,    0.6, 0.0, 0.0);
    HTSetConversion(context->conversions, HTML_LEVEL3_DOCUMENT,	"www/present",    HTExtParse,    0.9, 0.0, 0.0);
    HTSetConversion(context->conversions, HTML3_DOCUMENT,   	"www/present",    HTExtParse,    0.8, 0.0, 0.0);
    HTSetConversion(context->conversions, "application/octet-stream","www/present", HTSaveLocally, 0.1, 0.0, 0.0);
    HTSetConversion(context->conversions, "application/x-compressed","www/present", HTSaveLocally, 0.1, 0.0, 0.0);
    HTSetConversion(context->conversions, "*/*",            "www/present",    HTExtParse,    0.1, 0.0, 0.0); 

    if (!NoMailcap)
	register_mailcaps();

    if (CacheRoot)
	HTCache_enable(CacheRoot);
    else
	HTCache_disable();

}

void libExit(int pCache)
{
    HTList *l;
    Doc *d;
    HTParentAnchor *a;

    if (BRIDGE_TRACE)
        fprintf(stderr,"libExit cleaning up\n");

    /* delete file cache */

    if (!pCache)
        HTCacheDeleteAll();

    /* delete various other chunks -- purify, thanks! */

    HTList_free (reqlist);

    if (context) {
	l = context->registered_anchors;

#if 0
#ifndef DOCFREE_BUG
	/* for some reason, the fails on sgi's */

	while (a = (HTParentAnchor *)HTList_nextObject(l)) {
	    if (a->document)
		DocFree((Doc *)a->document, TRUE);
	}
#endif
#endif
	HTList_free (context->registered_anchors);
	HTList_free (context->conversions);
    }

    HTLibTerminate();
}
